import { FluentBundle, FluentVariable } from "@fluent/bundle";
import { ReactElement } from "react";
import { MarkupParser } from "./markup.js";
/**
 * `ReactLocalization` handles translation formatting and fallback.
 *
 * The current negotiated fallback chain of languages is stored in the
 * `ReactLocalization` instance in form of an iterable of `FluentBundle`
 * instances. This iterable is used to find the best existing translation for
 * a given identifier.
 *
 * The `ReactLocalization` class instances are exposed to `Localized` elements
 * via the `LocalizationProvider` component.
 */
export declare class ReactLocalization {
    bundles: Iterable<FluentBundle>;
    parseMarkup: MarkupParser | null;
    constructor(bundles: Iterable<FluentBundle>, parseMarkup?: MarkupParser | null);
    getBundle(id: string): FluentBundle | null;
    areBundlesEmpty(): boolean;
    getString(id: string, vars?: Record<string, FluentVariable> | null, fallback?: string): string;
    getElement(sourceElement: ReactElement, id: string, args?: {
        vars?: Record<string, FluentVariable>;
        elems?: Record<string, ReactElement>;
        attrs?: Record<string, boolean>;
    }): ReactElement;
    reportError(error: Error): void;
}
